import org.jetbrains.exposed.dao.*
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.`java-time`.*
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.transaction

/*
object MyTable0 : Table() {
    val id: Column<Int> = integer("id").autoIncrement()
    val textcol = varchar("textcol", length = 50).index()
    val doublecol = double("doublecol").check { it.between(0.0, 1.0) }
    val datecol = date("datecol").nullable()
    override val primaryKey = PrimaryKey(id)
}
*/

object MyTable : IntIdTable() {
    val textcol = varchar("textcol", length = 50).index()
    val doublecol = double("doublecol").check { it.between(0.0, 1.0) }
    val datecol = date("datecol").nullable()
    val defaultcol = integer("defaultcol").default(123)
    val ts = datetime("ts").defaultExpression(CurrentDateTime())
    val rnd = integer("rnd").clientDefault { (0..100).random() }
}
class MyRecord(id: EntityID<Int>) : IntEntity(id) {
    companion object : IntEntityClass<MyRecord>(MyTable)
    var textcol by MyTable.textcol      // String
    var doublecol by MyTable.doublecol // Double
    var datecol by MyTable.datecol   // LocalDate?
    var defaultcol by MyTable.defaultcol
    var ts by MyTable.ts
    var rnd by MyTable.rnd
}

fun testMyTableRecord() {
    transaction {
        // Logging
        addLogger(StdOutSqlLogger)

        // Tabelle erzeugen
        SchemaUtils.create(MyTable)

        // neuen Datensatz einfügen
        val newid = MyTable.insert {
            it[textcol] = "abc"
            it[doublecol] = 0.3
        }[MyTable.id]

        // noch ein Datensatz
        MyTable.insert {
            it[textcol] = "other"
            it[doublecol] = 0.3
        }

        // Schleife über alle Datensätze
        for (r in MyRecord.all().sortedBy { it.textcol }) {
            println("id=${r.id} col=${r.defaultcol} txt=${r.textcol}")
            println("ts=${r.ts}")
            println("rnd=${r.rnd}")
            if (r.id.value == 1)
                r.delete()
        }
        // delete testen
        MyTable.deleteWhere  { MyTable.id.greater(1) }
        println("nach delete")
        for (r in MyRecord.all().sortedBy { it.textcol }) {
            println("id=${r.id} col=${r.defaultcol} txt=${r.textcol}")
            println("ts=${r.ts}")
            println("rnd=${r.rnd}")
            if (r.id.value == 1)
                r.delete()
        }
    }
}