import org.jetbrains.exposed.dao.*
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.transaction
import java.sql.SQLException

// Singletons für die Tabellen (Plural)
object Publishers : IntIdTable() {
    val name = varchar("name", length = 100)
}
object Books : IntIdTable() {
    val title = varchar("title", 250)
    val publyear = integer("publyear")
    val publId = reference("publId", Publishers)
}
object Authors : IntIdTable() {
    val name = varchar("name", 100)
}
object BookAuthors : Table() {
    val bookId = reference("bookId", Books)
    val authorId = reference("AuthorId", Authors)
    override val primaryKey = PrimaryKey(bookId, authorId)
}


// Buchdatenbank einrichten
fun setupBooks() {
    transaction {
        // Logging
        // addLogger(StdOutSqlLogger)

        // Tabellen erzeugen
        for (t in listOf(Publishers, Authors, Books, BookAuthors))
            SchemaUtils.createMissingTablesAndColumns(t)

        // Verlage
        val publishers = listOf("ebooks.kofler", "Rheinwerk Verlag")
        val publIds = mutableMapOf<String, EntityID<Int>>()
        for (pname in publishers)
            publIds[pname] = Publishers.insert {
                it[name] = pname
            }[Publishers.id]

        // Autoren
        val authors = listOf(
            "Kofler, Michael", "Öggl, Bernd",
            "Kühnast, Charly", "Scherbeck, Christoph"
        )
        val authIds = mutableMapOf<String, EntityID<Int>>()
        for (aname in authors)
            authIds[aname] = Authors.insert {
                it[name] = aname
            }[Authors.id]

        // Kotlin-Buch
        var bkid = Books.insert {
            it[title] = "Kotlin -- Das umfassende Handbuch"
            it[publyear] = 2020
            it[publId] = publIds["Rheinwerk Verlag"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }

        // Linux-Buch
        bkid = Books.insert {
            it[title] = "Linux -- Das umfassende Handbuch"
            it[publyear] = 2019
            it[publId] = publIds["Rheinwerk Verlag"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }

        // Raspberry-Pi-Buch
        bkid = Books.insert {
            it[title] = "Raspberry Pi -- Das umfassende Handbuch"
            it[publyear] = 2019
            it[publId] = publIds["Rheinwerk Verlag"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Scherbeck, Christoph"]!!
        }
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kühnast, Charly"]!!
        }

        // Docker-Buch
        bkid = Books.insert {
            it[title] = "Docker -- Das Praxisbuch, 2. Aufl."
            it[publyear] = 2020
            it[publId] = publIds["Rheinwerk Verlag"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Öggl, Bernd"]!!
        }
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }

        // Ubuntu-E-Book
        bkid = Books.insert {
            it[title] = "Ubuntu 20.04"
            it[publyear] = 2020
            it[publId] = publIds["ebooks.kofler"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }
        // Pandoc-E-Book
        bkid = Books.insert {
            it[title] = "Markdown & Pandoc"
            it[publyear] = 2019
            it[publId] = publIds["ebooks.kofler"]!!
        }[Books.id]
        BookAuthors.insert {
            it[bookId] = bkid
            it[authorId] = authIds["Kofler, Michael"]!!
        }
    }
}
