import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.`java-time`.datetime
import org.jetbrains.exposed.sql.`java-time`.CurrentDateTime
import org.jetbrains.exposed.sql.transactions.transaction
import org.jetbrains.exposed.sql.transactions.transactionScope
import java.time.DateTimeException
import java.time.LocalDateTime

object Logins : IntIdTable() {
    val name = varchar("name", length = 100)
    val login = datetime("login").defaultExpression(CurrentDateTime())
    val logout = datetime("logout").nullable()
}

fun testDateTime() {
    transaction {
        addLogger(StdOutSqlLogger)
        SchemaUtils.create(Logins)
        Logins.insert { login ->
            login[Logins.name] = "Michael"
            login[logout] = LocalDateTime.now().plusHours(1L)
        }
        for (row in Logins.selectAll()) {
            println(row)
        }
        println("---")
        for (row in Logins.select { Logins.login lessEq LocalDateTime.now() }) {
            println(row)
        }
    }
}
/*
SQL code and output with H2 (as I expect)

SQL: CREATE TABLE IF NOT EXISTS LOGINS (ID INT AUTO_INCREMENT PRIMARY KEY, "NAME" VARCHAR(100) NOT NULL, LOGIN DATETIME DEFAULT (CURRENT_TIMESTAMP) NOT NULL, LOGOUT DATETIME NULL)
SQL: INSERT INTO LOGINS (LOGIN, LOGOUT, "NAME") VALUES (CURRENT_TIMESTAMP, '2020-03-05T17:43:58.505532', 'Michael')
SQL: SELECT LOGINS.ID, LOGINS."NAME", LOGINS.LOGIN, LOGINS.LOGOUT FROM LOGINS
Logins.id=1, Logins.name=Michael, Logins.login=2020-03-05 16:43:58.517831, Logins.logout=2020-03-05 17:43:58.505
---
SQL: SELECT LOGINS.ID, LOGINS."NAME", LOGINS.LOGIN, LOGINS.LOGOUT FROM LOGINS WHERE LOGINS.LOGIN <= '2020-03-05T16:43:58.526958'
Logins.id=1, Logins.name=Michael, Logins.login=2020-03-05 16:43:58.517831, Logins.logout=2020-03-05 17:43:58.505

SQL code and output with SQLite (wrong)

SQL: CREATE TABLE IF NOT EXISTS Logins (id INTEGER PRIMARY KEY AUTOINCREMENT, "name" VARCHAR(100) NOT NULL, login NUMERIC DEFAULT (CURRENT_TIMESTAMP) NOT NULL, logout NUMERIC NULL)
SQL: INSERT INTO Logins (login, logout, "name") VALUES (CURRENT_TIMESTAMP, '2020-03-05T17:45:25.750784', 'Michael')
SQL: SELECT Logins.id, Logins."name", Logins.login, Logins.logout FROM Logins
Logins.id=1, Logins.name=Michael, Logins.login=2020-03-05 15:45:25, Logins.logout=1583426725750
  (Logins.logout is wrong)
---
SQL: SELECT Logins.id, Logins."name", Logins.login, Logins.logout FROM Logins WHERE Logins.login <= '2020-03-05T16:45:25.765578'
  (no result, wrong)

 */

