import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.TransactionManager
import org.jetbrains.exposed.sql.transactions.transaction
import java.io.File
import java.sql.Connection

object Artists : IntIdTable() {
    val name = varchar("name", length = 100)
}
object Albums : IntIdTable() {
    val title = varchar("title", 250)
    val artistId = reference("artistId", Artists)
    // onDelete = ReferenceOption.CASCADE,
    // onUpdate = ReferenceOption.CASCADE)
}

fun testSQLiteFK() {
    // File("fktest.db").delete()
    // Database.connect("jdbc:sqlite:fktest.db", "org.sqlite.JDBC")
    TransactionManager.manager.defaultIsolationLevel =
       Connection.TRANSACTION_SERIALIZABLE

    transaction {
        addLogger(StdOutSqlLogger)
        SchemaUtils.create(Artists)
        SchemaUtils.create(Albums)
        val pfId = Artists.insertAndGetId { it[name] = "Pink Floyd" }
        Albums.insert {
            it[title] = "Dark side of the moon"
            it[artistId] = pfId
        }
    }
    transaction {
        addLogger(StdOutSqlLogger)

        // explicitly enabling foreign keys (?)
        exec("PRAGMA foreign_keys = ON")

        // should fail because of FK constraint
        val n = Artists.deleteWhere { Artists.name eq "Pink Floyd" }
        println("$n records deleted")
    }
    transaction {
        addLogger(StdOutSqlLogger)




        for (row in Artists.selectAll())
            println("Artist: $row")
        // no output

        for (row in Albums.selectAll())
            println("Album: $row")
        // Albums.id=1, Albums.title=Dark side of the moon, Albums.artistId=1
        // refers to no longer existing artist
    }
}