
import com.zaxxer.hikari.HikariConfig
import com.zaxxer.hikari.HikariDataSource
import org.jetbrains.exposed.dao.IntEntity
import org.jetbrains.exposed.dao.IntEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IntIdTable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.`java-time`.CurrentDateTime
import org.jetbrains.exposed.sql.`java-time`.datetime
import org.jetbrains.exposed.sql.transactions.TransactionManager
import org.jetbrains.exposed.sql.transactions.transaction
import java.sql.Connection


fun main() {
    // H2 file
    // val db = Database.connect("jdbc:h2:./myh2.db", "org.h2.Driver")
    // H2 in-memory
    // val db = Database.connect("jdbc:h2:mem:test;DB_CLOSE_DELAY=-1", "org.h2.Driver")

    // SQLite file
    val db = Database.connect("jdbc:sqlite:my.db", "org.sqlite.JDBC")
    // SQLite in-memory. Achtung, mit dem Ende jeder *Transaktion* gehen alle Daten verloren.
    // val db = Database.connect("jdbc:sqlite::memory:", "org.sqlite.JDBC")


    // MySQL
    // jdbc:mysql://<host>[:port]/<db>
    // val db = Database.connect("jdbc:mysql://localhost/test1",
    //    driver = "com.mysql.cj.jdbc.Driver",
    //    user = "test1",
    //    password = "geheim")

    // Hikari Connection Pool für MySQL
    //     val cfg: HikariConfig = HikariConfig().apply {
    //         jdbcUrl = "jdbc:mysql://localhost/test1"
    //         driverClassName = "com.mysql.cj.jdbc.Driver"
    //         username = "test1"
    //         password = "geheim"
    //         maximumPoolSize = 12
    //     }
    //     val dataSource = HikariDataSource(cfg)
    //     val db = Database.connect(dataSource)


    transaction {
        // Zum Debugging: SQL-Statements in der Konsole
        // protokollieren
        addLogger(StdOutSqlLogger)

        // Tabelle erzeugen (falls noch nicht vorhanden)
        SchemaUtils.create(RunLogs)


        // Programmstatus in Tabelle speichern
        val newid = RunLogs.insert {
            it[status] = "started"
            // it[ts] = LocalDateTime.now()
        }[RunLogs.id]

        // SELECT * FROM RunLug
        val entries = RunLog.all().sortedBy { it.id }
        for (entry in entries) {
            println("ID=${entry.id} status=${entry.status} timestamp=${entry.ts}")
        }
    }

    // Thread.sleep(500L)  // 500 ms

    transaction {
        RunLogs.insert {
            it[status] = "ending"
            // it[ts] = LocalDateTime.now()
        }
    }
}

object RunLogs: IntIdTable() {
    // Datentyp Column<String>
    val status = varchar("status", 50)
    // Datentyp Column<LocalDateTime>
    val ts = datetime("ts").defaultExpression(CurrentDateTime())
    // alternativ, wenn Datebank keine autom. TIMESTAMPs unterstützt
    // .clientDefault { LocalDateTime.now() }
}

class RunLog(id: EntityID<Int>): IntEntity(id) {
    companion object : IntEntityClass<RunLog>(RunLogs)
    var status by RunLogs.status
    var ts by RunLogs.ts
}